@@ -1,5 +1,6 @@
+// $OpenLDAP$
 /*
- * Copyright 2000, OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 2000-2019 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -39,10 +40,6 @@ LDAPModifyRequest::~LDAPModifyRequest(){
     DEBUG(LDAP_DEBUG_DESTROY, 
             "LDAPModifyRequest::~LDAPModifyRequest()" << endl);
     delete m_modList;
-    // flush this entry from cache.
-    //m_connection->uncache_entry(m_dn);
-    // I think we need to do this... (j.costlow)
-    m_connection->flush_cache();
 }
 
 LDAPMessageQueue* LDAPModifyRequest::sendRequest(){
