@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2011 The OpenLDAP Foundation.
+ * Copyright 2000-2019 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -101,6 +101,8 @@ static void * bdb_tool_index_task( void *ctx, void *ptr );
 static int
 bdb_tool_entry_get_int( BackendDB *be, ID id, Entry **ep );
 
+static int bdb_tool_threads;
+
 int bdb_tool_entry_open(
 	BackendDB *be, int mode )
 {
@@ -138,14 +140,17 @@ int bdb_tool_entry_open(
 			ldap_pvt_thread_cond_init( &bdb_tool_index_cond_work );
 			if ( bdb->bi_nattrs ) {
 				int i;
-				bdb_tool_index_threads = ch_malloc( slap_tool_thread_max * sizeof( int ));
-				bdb_tool_index_rec = ch_malloc( bdb->bi_nattrs * sizeof( IndexRec ));
-				bdb_tool_index_tcount = slap_tool_thread_max - 1;
-				for (i=1; i<slap_tool_thread_max; i++) {
-					int *ptr = ch_malloc( sizeof( int ));
-					*ptr = i;
-					ldap_pvt_thread_pool_submit( &connection_pool,
-						bdb_tool_index_task, ptr );
+				bdb_tool_threads = slap_tool_thread_max - 1;
+				if ( bdb_tool_threads > 1 ) {
+					bdb_tool_index_threads = ch_malloc( bdb_tool_threads * sizeof( int ));
+					bdb_tool_index_rec = ch_malloc( bdb->bi_nattrs * sizeof( IndexRec ));
+					bdb_tool_index_tcount = bdb_tool_threads - 1;
+					for (i=1; i<bdb_tool_threads; i++) {
+						int *ptr = ch_malloc( sizeof( int ));
+						*ptr = i;
+						ldap_pvt_thread_pool_submit( &connection_pool,
+							bdb_tool_index_task, ptr );
+					}
 				}
 			}
 			bdb_tool_info = bdb;
@@ -174,29 +179,31 @@ int bdb_tool_entry_close(
 					&bdb_tool_trickle_mutex );
 		ldap_pvt_thread_mutex_unlock( &bdb_tool_trickle_mutex );
 #endif
-		ldap_pvt_thread_mutex_lock( &bdb_tool_index_mutex );
+		if ( bdb_tool_threads > 1 ) {
+			ldap_pvt_thread_mutex_lock( &bdb_tool_index_mutex );
 
-		/* There might still be some threads starting */
-		while ( bdb_tool_index_tcount ) {
-			ldap_pvt_thread_cond_wait( &bdb_tool_index_cond_main,
+			/* There might still be some threads starting */
+			while ( bdb_tool_index_tcount > 0 ) {
+				ldap_pvt_thread_cond_wait( &bdb_tool_index_cond_main,
+						&bdb_tool_index_mutex );
+			}
+
+			bdb_tool_index_tcount = bdb_tool_threads - 1;
+			ldap_pvt_thread_cond_broadcast( &bdb_tool_index_cond_work );
+
+			/* Make sure all threads are stopped */
+			while ( bdb_tool_index_tcount > 0 ) {
+				ldap_pvt_thread_cond_wait( &bdb_tool_index_cond_main,
 					&bdb_tool_index_mutex );
+			}
+			ldap_pvt_thread_mutex_unlock( &bdb_tool_index_mutex );
+
+			ch_free( bdb_tool_index_threads );
+			ch_free( bdb_tool_index_rec );
+			bdb_tool_index_tcount = bdb_tool_threads - 1;
 		}
-
-		bdb_tool_index_tcount = slap_tool_thread_max - 1;
-		ldap_pvt_thread_cond_broadcast( &bdb_tool_index_cond_work );
-
-		/* Make sure all threads are stopped */
-		while ( bdb_tool_index_tcount ) {
-			ldap_pvt_thread_cond_wait( &bdb_tool_index_cond_main,
-				&bdb_tool_index_mutex );
-		}
-		ldap_pvt_thread_mutex_unlock( &bdb_tool_index_mutex );
-
 		bdb_tool_info = NULL;
 		slapd_shutdown = 0;
-		ch_free( bdb_tool_index_threads );
-		ch_free( bdb_tool_index_rec );
-		bdb_tool_index_tcount = slap_tool_thread_max - 1;
 	}
 
 	if( eh.bv.bv_val ) {
@@ -580,7 +587,7 @@ bdb_tool_index_add(
 	if ( !bdb->bi_nattrs )
 		return 0;
 
-	if ( slapMode & SLAP_TOOL_QUICK ) {
+	if ( bdb_tool_threads > 1 ) {
 		IndexRec *ir;
 		int i, rc;
 		Attribute *a;
@@ -598,20 +605,20 @@ bdb_tool_index_add(
 		bdb_tool_ix_op = op;
 		ldap_pvt_thread_mutex_lock( &bdb_tool_index_mutex );
 		/* Wait for all threads to be ready */
-		while ( bdb_tool_index_tcount ) {
+		while ( bdb_tool_index_tcount > 0 ) {
 			ldap_pvt_thread_cond_wait( &bdb_tool_index_cond_main, 
 				&bdb_tool_index_mutex );
 		}
-		for ( i=1; i<slap_tool_thread_max; i++ )
+		for ( i=1; i<bdb_tool_threads; i++ )
 			bdb_tool_index_threads[i] = LDAP_BUSY;
-		bdb_tool_index_tcount = slap_tool_thread_max - 1;
+		bdb_tool_index_tcount = bdb_tool_threads - 1;
 		ldap_pvt_thread_cond_broadcast( &bdb_tool_index_cond_work );
 		ldap_pvt_thread_mutex_unlock( &bdb_tool_index_mutex );
 		rc = bdb_index_recrun( op, bdb, ir, e->e_id, 0 );
 		if ( rc )
 			return rc;
 		ldap_pvt_thread_mutex_lock( &bdb_tool_index_mutex );
-		for ( i=1; i<slap_tool_thread_max; i++ ) {
+		for ( i=1; i<bdb_tool_threads; i++ ) {
 			if ( bdb_tool_index_threads[i] == LDAP_BUSY ) {
 				ldap_pvt_thread_cond_wait( &bdb_tool_index_cond_main, 
 					&bdb_tool_index_mutex );
@@ -665,6 +672,8 @@ ID bdb_tool_entry_put(
 			 text->bv_val, 0, 0 );
 		return NOID;
 	}
+	Debug( LDAP_DEBUG_TRACE, LDAP_XSTRING(bdb_tool_entry_put) ": txn id: %x\n",
+		tid->id(tid), 0, 0 );
 	}
 
 	op.o_hdr = &ohdr;
@@ -823,6 +832,11 @@ int bdb_tool_entry_reindex(
 		return -1;
 	}
 
+	op.o_hdr = &ohdr;
+	op.o_bd = be;
+	op.o_tmpmemctx = NULL;
+	op.o_tmpmfuncs = &ch_mfuncs;
+
 	if (! (slapMode & SLAP_TOOL_QUICK)) {
 	rc = TXN_BEGIN( bi->bi_dbenv, NULL, &tid, bi->bi_db_opflags );
 	if( rc != 0 ) {
@@ -832,6 +846,8 @@ int bdb_tool_entry_reindex(
 			db_strerror(rc), rc, 0 );
 		goto done;
 	}
+	Debug( LDAP_DEBUG_TRACE, LDAP_XSTRING(bdb_tool_entry_reindex) ": txn id: %x\n",
+		tid->id(tid), 0, 0 );
 	}
  	
 	/*
@@ -845,11 +861,6 @@ int bdb_tool_entry_reindex(
 		"=> " LDAP_XSTRING(bdb_tool_entry_reindex) "( %ld, \"%s\" )\n",
 		(long) id, e->e_dn, 0 );
 
-	op.o_hdr = &ohdr;
-	op.o_bd = be;
-	op.o_tmpmemctx = NULL;
-	op.o_tmpmfuncs = &ch_mfuncs;
-
 	rc = bdb_tool_index_add( &op, tid, e );
 
 done:
@@ -922,6 +933,8 @@ ID bdb_tool_entry_modify(
 				 text->bv_val, 0, 0 );
 			return NOID;
 		}
+		Debug( LDAP_DEBUG_TRACE, LDAP_XSTRING(bdb_tool_entry_modify) ": txn id: %x\n",
+			tid->id(tid), 0, 0 );
 	}
 
 	op.o_hdr = &ohdr;
