@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/search.c,v 1.117.2.18 2011/01/26 23:23:34 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2011 The OpenLDAP Foundation.
+ * Copyright 1999-2019 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Masarati.
  * Portions Copyright 2004 Mark Adamson.
@@ -2460,6 +2460,7 @@ backsql_search( Operation *op, SlapReply *rs )
 				goto end_of_search;
 
 			case LDAP_SIZELIMIT_EXCEEDED:
+			case LDAP_BUSY:
 				goto send_results;
 			}
 		}
