@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2018 The OpenLDAP Foundation.
+ * Copyright 1998-2019 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -323,7 +323,7 @@ slap_auxprop_lookup(
 
 	/* we don't know anything about this, ignore it */
 	if ( !conn ) {
-		rc == LDAP_SUCCESS;
+		rc = LDAP_SUCCESS;
 		goto done;
 	}
 
